@@ -5,22 +5,22 @@
 
 ### 0) Generic covariance function
 
-##' Correlation function of selected type, supporting both isotropic and product forms
-##' @param X1 matrix of design locations, one point per row
-##' @param X2 matrix of design locations if correlation is calculated between \code{X1} and \code{X2} (otherwise calculated between \code{X1} and itself)
-##' @param theta vector of lengthscale parameters (either of size one if isotropic or of size d if anisotropic)
-##' @param type one of "\code{Gaussian}", "\code{Matern5_2}", "\code{Matern3_2}"
-##' @export
-##' @details 
-##' Definition of univariate correlation function and hyperparameters:
-##' \itemize{
-##' \item "\code{Gaussian}": \eqn{c(x, y) = exp(-(x-y)^2/theta)}
-##' \item "\code{Matern5_2}": \eqn{c(x, y) = (1+sqrt(5)/theta * abs(x-y) + 5/(3*theta^2)(x-y)^2) * exp(-sqrt(5)*abs(x-y)/theta)}
-##' \item "\code{Matern3_2}": \eqn{c(x, y) = (1+sqrt(3)/theta * abs(x-y)) * exp(-sqrt(3)*abs(x-y)/theta)}
-##' }
-##' Multivariate correlations are product of univariate ones.
-##' @useDynLib hetGP
-##' @importFrom  Rcpp evalCpp
+#' Correlation function of selected type, supporting both isotropic and product forms
+#' @param X1 matrix of design locations, one point per row
+#' @param X2 matrix of design locations if correlation is calculated between \code{X1} and \code{X2} (otherwise calculated between \code{X1} and itself)
+#' @param theta vector of lengthscale parameters (either of size one if isotropic or of size d if anisotropic)
+#' @param type one of "\code{Gaussian}", "\code{Matern5_2}", "\code{Matern3_2}"
+#' @export
+#' @details 
+#' Definition of univariate correlation function and hyperparameters:
+#' \itemize{
+#' \item "\code{Gaussian}": \eqn{c(x, y) = exp(-(x-y)^2/theta)}
+#' \item "\code{Matern5_2}": \eqn{c(x, y) = (1+sqrt(5)/theta * abs(x-y) + 5/(3*theta^2)(x-y)^2) * exp(-sqrt(5)*abs(x-y)/theta)}
+#' \item "\code{Matern3_2}": \eqn{c(x, y) = (1+sqrt(3)/theta * abs(x-y)) * exp(-sqrt(3)*abs(x-y)/theta)}
+#' }
+#' Multivariate correlations are product of univariate ones.
+#' @useDynLib hetGP
+#' @importFrom  Rcpp evalCpp
 cov_gen <- function(X1, X2 = NULL, theta, type = c("Gaussian", "Matern5_2", "Matern3_2")){
   type <- match.arg(type)
   if(type == "Gaussian")
@@ -63,6 +63,8 @@ partial_cov_gen <- function(X1, theta, type = "Gaussian", arg, ..., X2 = NULL){
         return(partial_d_C_Matern3_2_dtheta_k(X1 = X1, theta = theta, ...))
       if(arg == "k_theta_g")
         return(partial_d_Cg_Matern3_2_d_k_theta_g(X1 = X1, theta = theta, ... ))
+      if(arg == "X_i_j")
+        return(partial_d_C_Matern3_2_dX_i_j(X1 = X1, theta = theta, ... ))
     }
   }else{
     if(type == "Gaussian"){
@@ -86,6 +88,8 @@ partial_cov_gen <- function(X1, theta, type = "Gaussian", arg, ..., X2 = NULL){
         return(partial_d_k_Matern3_2_dtheta_k(X1 = X1, X2 = X2, theta = theta, ...))
       if(arg == "k_theta_g")
         return(partial_d_kg_Matern3_2_d_k_theta_g(X1 = X1, X2 = X2, theta = theta, ... ))
+      if(arg == "X_i_j")
+        return(partial_d_k_Matern3_2_dX_i_j(X1 = X1, X2 = X2, theta = theta, ... ))
     }
   }
 }
@@ -292,4 +296,35 @@ partial_d_kg_Matern3_2_d_k_theta_g <- function(X1, X2, theta, k_theta_g){
   return(d_matern3_2_2args_kthetag(X1 * rep(1/theta, rep(nrow(X1),length(theta))), X2 * rep(1/theta, rep(nrow(X2),length(theta))), k_theta_g))
 }
 
+## Derivative with respect to X[i,j]. Useful for pseudo inputs, to be multiplied by the covariance matrix
+## @param i1 row
+## @param i2 column
+partial_d_C_Matern3_2_dX_i_j <- function(X1, theta, i1, i2){
+  
+  ## 1-dimensional/isotropic case
+  if(length(theta) == 1){
+    tmp <- partial_d_dist_abs_dX_i1_i2_m32(X1/theta, i1, i2)
+    return(tmp/theta)
+  }
+  
+  tmp <- partial_d_dist_abs_dX_i1_i2_m32(X1/theta[i2], i1, i2)
+  return(tmp / theta[i2])
+}
+
+## Derivative with respect to X[i,j]. Useful for pseudo inputs, to be multiplied by the covariance matrix
+## @param i1 row
+## @param i2 column
+## @param theta lengthscales
+partial_d_k_Matern3_2_dX_i_j <- function(X1, X2, theta, i1, i2){
+  
+  ## 1-dimensional/isotropic case
+  if(length(theta) == 1){
+    tmp <- partial_d_dist_abs_dX1_i1_i2_X2_m32(X1/theta, X2/theta, i1, i2)
+    return(tmp/theta)
+  }
+  
+  tmp <- partial_d_dist_abs_dX1_i1_i2_X2_m32(X1/theta[i2], X2/theta[i2], i1, i2)
+  return(tmp / theta[i2])
+}
+
 
